@@ -1,4 +1,4 @@
-/*	SCCS Id: @(#)edog.h	3.1	93/01/16	*/
+/*	SCCS Id: @(#)edog.h	3.3	97/10/23	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -7,10 +7,10 @@
 
 /*	various types of food, the lower, the better liked.	*/
 
-#define DOGFOOD	0
-#define CADAVER	1
-#define ACCFOOD	2
-#define MANFOOD	3
+#define DOGFOOD 0
+#define CADAVER 1
+#define ACCFOOD 2
+#define MANFOOD 3
 #define APPORT	4
 #define POISON	5
 #define UNDEF	6
@@ -19,10 +19,13 @@
 struct edog {
 	long droptime;			/* moment dog dropped object */
 	unsigned dropdist;		/* dist of drpped obj from @ */
-	unsigned apport;		/* amount of training */
+	int apport;			/* amount of training */
 	long whistletime;		/* last time he whistled */
 	long hungrytime;		/* will get hungry at this time */
 	coord ogoal;			/* previous goal location */
+	int abuse;			/* track abuses to this pet */
+	int revivals;			/* count pet deaths */
+	Bitfield(killed_by_u, 1);	/* you attempted to kill him */
 };
 #define EDOG(mon)	((struct edog *)&(mon)->mextra[0])
 
