@@ -1,14 +1,10 @@
-/*	SCCS Id: @(#)epri.h	3.2	90/02/22	*/
+/*	SCCS Id: @(#)epri.h	3.3	97/05/01	*/
 /* Copyright (c) Izchak Miller, 1989.				  */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #ifndef EPRI_H
 #define EPRI_H
 
-#ifndef DUNGEON_H
-#include "dungeon.h"	/* nested #include "align.h" */
-#endif
-
 struct epri {
 	aligntyp shralign;	/* alignment of priest's shrine */
 				/* leave as first field to match emin */
