@@ -1,13 +1,13 @@
-/*	SCCS Id: @(#)exper.c	3.2	96/06/16	*/
+/*	SCCS Id: @(#)exper.c	3.3	97/01/29	*/
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "hack.h"
 
-static long FDECL(newuexp, (int));
-static int FDECL(enermod, (int));
+STATIC_DCL long FDECL(newuexp, (int));
+STATIC_DCL int FDECL(enermod, (int));
 
-static long
+STATIC_OVL long
 newuexp(lev)
 int lev;
 {
@@ -16,14 +16,23 @@ int lev;
 	return (10000000L * ((long)(lev - 19)));
 }
 
-static int
+STATIC_OVL int
 enermod(en)
 int en;
 {
-	if(Role_is('W') || Role_is('P')) return(2 * en);
-	else if(Role_is('H') || Role_is('K')) return((3 * en) / 2);
-	else if(Role_is('B') || Role_is('V')) return((3 * en) / 4);
-	else return(en);
+	switch (Role_switch) {
+	case PM_PRIEST:
+	case PM_WIZARD:
+	    return(2 * en);
+	case PM_HEALER:
+	case PM_KNIGHT:
+	    return((3 * en) / 2);
+	case PM_BARBARIAN:
+	case PM_VALKYRIE:
+	    return((3 * en) / 4);
+	default:
+	    return (en);
+	}
 }
 
 int
@@ -59,19 +68,20 @@ experience(mtmp, nk)	/* return # of exp points for mtmp after nk killed */
 
 /*	For each "special" damage type give extra experience */
 	for(i = 0; i < NATTK; i++) {
-
 	    tmp2 = ptr->mattk[i].adtyp;
 	    if(tmp2 > AD_PHYS && tmp2 < AD_BLND) tmp += 2*mtmp->m_lev;
-	    else if((tmp2 == AD_DRLI) || (tmp2 == AD_STON)) tmp += 50;
+	    else if((tmp2 == AD_DRLI) || (tmp2 == AD_STON) ||
+	    		(tmp2 == AD_SLIM)) tmp += 50;
 	    else if(tmp != AD_PHYS) tmp += mtmp->m_lev;
 		/* extra heavy damage bonus */
 	    if((int)(ptr->mattk[i].damd * ptr->mattk[i].damn) > 23)
 		tmp += mtmp->m_lev;
+	    if (tmp2 == AD_WRAP && ptr->mlet == S_EEL && !Amphibious)
+		tmp += 1000;
 	}
 
 /*	For certain "extra nasty" monsters, give even more */
 	if (extra_nasty(ptr)) tmp += (7 * mtmp->m_lev);
-	if (ptr->mlet == S_EEL && !Amphibious) tmp += 1000;
 
 /*	For higher level monsters, an additional bonus is given */
 	if(mtmp->m_lev > 8) tmp += 50;
@@ -95,7 +105,7 @@ more_experienced(exp, rexp)
 	   || flags.showscore
 #endif
 	   ) flags.botl = 1;
-	if (u.urexp >= (Role_is('W') ? 1000 : 2000))
+	if (u.urexp >= (Role_if(PM_WIZARD) ? 1000 : 2000))
 		flags.beginner = 0;
 }
 
@@ -107,7 +117,7 @@ losexp()		/* hit by drain life attack */
 	if (resists_drli(&youmonst)) return;
 
 	if(u.ulevel > 1) {
-		pline("Goodbye level %d.", u.ulevel--);
+		pline("%s level %d.", Goodbye(), u.ulevel--);
 		/* remove intrinsic abilities */
 		adjabil(u.ulevel + 1, u.ulevel);
 		reset_rndmonst(NON_PM);	/* new monster selection */
@@ -116,7 +126,13 @@ losexp()		/* hit by drain life attack */
 	num = newhp();
 	u.uhp -= num;
 	u.uhpmax -= num;
-	num = enermod(rn1(u.ulevel/2 + 1, 2));		/* M. Stephenson */
+	if (u.ulevel < urole.xlev)
+	    num = rn1(u.ulevel/2 + urole.enadv.lornd + urace.enadv.lornd,
+	    		urole.enadv.lofix + urace.enadv.lofix);
+	else
+	    num = rn1(u.ulevel/2 + urole.enadv.hirnd + urace.enadv.hirnd,
+	    		urole.enadv.hifix + urace.enadv.hifix);
+	num = enermod(num);		/* M. Stephenson */
 	u.uen -= num;
 	if (u.uen < 0)		u.uen = 0;
 	u.uenmax -= num;
@@ -134,43 +150,41 @@ losexp()		/* hit by drain life attack */
 void
 newexplevel()
 {
-	register int tmp;
-
-	if(u.ulevel < MAXULEV && u.uexp >= newuexp(u.ulevel)) {
-
-		u.ulevel++;
-		if (u.uexp >= newuexp(u.ulevel)) u.uexp = newuexp(u.ulevel) - 1;
-		pline("Welcome to experience level %d.", u.ulevel);
-		/* give new intrinsics */
-		adjabil(u.ulevel - 1, u.ulevel);
-		reset_rndmonst(NON_PM);	/* new monster selection */
-		tmp = newhp();
-		u.uhpmax += tmp;
-		u.uhp += tmp;
-		tmp = enermod(rn1((int)ACURR(A_WIS)/2+1, 2)); /* M. Stephenson */
-		u.uenmax += tmp;
-		u.uen += tmp;
-		flags.botl = 1;
-	}
+	if (u.ulevel < MAXULEV && u.uexp >= newuexp(u.ulevel))
+	    pluslvl(TRUE);
 }
 
 void
-pluslvl()
-{
+pluslvl(incr)
+boolean incr;	/* true iff via incremental experience growth */
+{		/*	(false for potion of gain level)      */
 	register int num;
 
-	You_feel("more experienced.");
+	if (!incr) You_feel("more experienced.");
 	num = newhp();
 	u.uhpmax += num;
 	u.uhp += num;
-	num = enermod(rn1((int)ACURR(A_WIS)/2+1, 2));	/* M. Stephenson */
+	if (u.ulevel < urole.xlev)
+	    num = rn1((int)ACURR(A_WIS)/2 + urole.enadv.lornd + urace.enadv.lornd,
+	    		urole.enadv.lofix + urace.enadv.lofix);
+	else
+	    num = rn1((int)ACURR(A_WIS)/2 + urole.enadv.hirnd + urace.enadv.hirnd,
+	    		urole.enadv.hifix + urace.enadv.hifix);
+	num = enermod(num);	/* M. Stephenson */
 	u.uenmax += num;
 	u.uen += num;
-	if(u.ulevel < MAXULEV) {
+	if (u.ulevel < MAXULEV) {
+	    if (incr) {
+		long tmp = newuexp(u.ulevel + 1);
+		if (u.uexp >= tmp) u.uexp = tmp - 1;
+	    } else {
 		u.uexp = newuexp(u.ulevel);
-		pline("Welcome to experience level %d.", ++u.ulevel);
-		adjabil(u.ulevel - 1, u.ulevel);
-		reset_rndmonst(NON_PM);	/* new monster selection */
+	    }
+	    ++u.ulevel;
+	    if (u.ulevelmax < u.ulevel) u.ulevelmax = u.ulevel;
+	    pline("Welcome to experience level %d.", u.ulevel);
+	    adjabil(u.ulevel - 1, u.ulevel);	/* give new intrinsics */
+	    reset_rndmonst(NON_PM);		/* new monster selection */
 	}
 	flags.botl = 1;
 }
@@ -178,15 +192,14 @@ pluslvl()
 long
 rndexp()
 {
-	register long minexp,maxexp;
+	long minexp, maxexp, diff, factor;
 
-	if(u.ulevel == 1)
-		return rn2((int)newuexp(1));
-	else {
-		minexp = newuexp(u.ulevel - 1);
-		maxexp = newuexp(u.ulevel);
-		return(minexp + rn2((int)(maxexp - minexp)));
-	}
+	minexp = (u.ulevel == 1) ? 0L : newuexp(u.ulevel - 1);
+	maxexp = newuexp(u.ulevel);
+	diff = maxexp - minexp,  factor = 1L;
+	while (diff >= (long)LARGEST_INT)
+	    diff /= 2L,  factor *= 2L;
+	return minexp + factor * (long)rn2((int)diff);
 }
 
 /*exper.c*/
